﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace IdentityAtRest.Utility.Identity
{
    public static class AuthorizationExtensions
    {
        public static bool IsAuthorized(this IEnumerable<IAuthorization> authorizations, string requestedPermission)
        {
            IAuthorization auth = GetAuthorization(authorizations, requestedPermission);
            if (auth != null)
                return auth.Authorized;
            return false;
        }

        public static bool IsAuthorizedForAll(this IList<IAuthorization> authorizations)
        {
            if (authorizations == null || authorizations.Count < 1)
                return false;

            foreach (IAuthorization auth in authorizations)
            {
                if (!auth.Authorized)
                    return false;
            }
            return true;
        }

        public static bool IsAuthorizedForAny(this IList<IAuthorization> authorizations)
        {
            if (authorizations == null || authorizations.Count < 1)
                return false;

            foreach (IAuthorization auth in authorizations)
            {
                if (auth.Authorized)
                    return true;
            }
            return false;
        }

        public static IAuthorization GetAuthorization(this IEnumerable<IAuthorization> authorizations,
                                                      string requestedPermission)
        {
            foreach (IAuthorization auth in authorizations)
            {
                if (CaseInsensitiveComparer.Default.Compare(auth.Permission, requestedPermission) == 0)
                    return auth;
            }
            return null;
        }

        public static void AddAuthorization(this IList<IAuthorization> authorizations, IAuthorization authorization)
        {
            if (authorizations == null)
                throw new ArgumentNullException("authorizations");
            if (authorization == null)
                throw new ArgumentNullException("authorization");

            authorizations.Add(authorization);
        }

        public static IAuthorization AddAuthorization(this IList<IAuthorization> authorizations, string permission,
                                                      bool isAuthorized)
        {
            return AddAuthorization(authorizations, permission, isAuthorized, "");
        }

        public static IAuthorization AddAuthorization(this IList<IAuthorization> authorizations, string permission,
                                                      bool isAuthorized, string explanation)
        {
            var auth = new DefaultAuthorization(permission, isAuthorized, explanation);
            AddAuthorization(authorizations, auth);
            return auth;
        }
    }
}